"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CacheAccount = void 0;
const web3_js_1 = require("@solana/web3.js");
class CacheAccount {
    constructor(client, address, state, _onStateUpdate) {
        this.client = client;
        this.address = address;
        this.state = state;
        this._onStateUpdate = _onStateUpdate;
        _onStateUpdate && this.subscribe();
    }
    static async create(client, clearing, authority) {
        const cacheAccountFull = web3_js_1.Keypair.generate();
        const preIx = await client.accounts.cacheAccount.createInstruction(cacheAccountFull, 65576);
        const ix = await client.methods
            .initCacheAccount()
            .accountsStrict({
            clearing,
            cacheAccount: cacheAccountFull.publicKey,
            authority
        })
            .instruction();
        return {
            cacheAccount: cacheAccountFull.publicKey,
            ixs: [preIx, ix],
            signers: [cacheAccountFull]
        };
    }
    static async load(client, address, onStateUpdateHandler) {
        const state = (await client.accounts.cacheAccount.fetchNullable(address));
        return new CacheAccount(client, address, state, onStateUpdateHandler);
    }
    getCaches() {
        const caches = [];
        for (const cache of this.state.caches) {
            if (!cache.oracleProducts.equals(web3_js_1.PublicKey.default)) {
                caches.push(cache);
            }
        }
        return caches;
    }
    getCache(idx) {
        return this.state.caches[idx];
    }
    subscribe() {
        this.client.accounts.cacheAccount
            .subscribe(this.address)
            .on('change', (state) => {
            this.state = state;
            // todo: check if dexMarkets need to be reloaded.(market listing/delisting)
            if (this._onStateUpdate) {
                this._onStateUpdate(this.state);
            }
        });
    }
    async unsubscribe() {
        await this.client.accounts.cacheAccount.unsubscribe(this.address);
    }
}
exports.CacheAccount = CacheAccount;
//# sourceMappingURL=cacheAccount.js.map